#
#  Copyright © 2019 Collabora Ltd.
#
#  Permission is hereby granted, free of charge, to any person obtaining
#  a copy of this software and associated documentation files (the
#  "Software"), to deal in the Software without restriction, including
#  without limitation the rights to use, copy, modify, merge, publish,
#  distribute, sublicense, and/or sell copies of the Software, and to
#  permit persons to whom the Software is furnished to do so, subject to
#  the following conditions:
#
#  The above copyright notice and this permission notice (including the
#  next paragraph) shall be included in all copies or substantial
#  portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
#  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
#  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
#  NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
#  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
#  ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
#  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#  SOFTWARE.
#
# Usage: source this script then 'display_flight_rec'
#

import gdb

class DisplayFlightRecorder(gdb.Command):
    def __init__(self):

        self.rb = ''
        symbol_found = False

        ring_buff =  gdb.lookup_global_symbol("weston_primary_flight_recorder_ring_buffer")
        if ring_buff == None:
            print("'weston_ring_buffer' symbol not found!")
            print("Either weston is too old or weston hasn't been loaded in memory")
        else:
            self.rb = ring_buff
            self.display_rb_data(self.rb)
            symbol_found = True

        if symbol_found:
            super(DisplayFlightRecorder, self).__init__("display_flight_rec",
                                                        gdb.COMMAND_DATA)

    def display_rb_data(self, rb):
        print("Flight recorder data found. Use 'display_flight_rec' "
                "to display its contents")
        # display this data (only) if symbol is not empty (happens if the program is not ran at all)
        if rb.value():
            print("Data at byte {append}, Size: {size}B, "
                  "Overlapped: {overlap}".format(append=rb.value()['append_pos'],
                                                size=rb.value()['size'],
                                                overlap=rb.value()['overlap']))

    # poor's man fwrite()
    def gen_contents(self, start, stop):
        _str = ''
        for j in range(start, stop):
            _str += chr(self.rb.value()['buf'][j])
        return _str

    # mirrors C version, as to make sure we're not reading other parts...
    def display_flight_rec_contents(self):

        # symbol is there but not loaded, we're not far enough
        if self.rb.value() == 0x0:
            print("Flight recorder found, but not loaded yet!")
            return
        else:
            print("Displaying flight recorder contents:")

        append_pos = self.rb.value()['append_pos']
        size = self.rb.value()['size']
        overlap = self.rb.value()['overlap']

        # if we haven't overflown and we're still at 0 means
        # we still aren't far enough to be populated
        if append_pos == 0 and not overlap:
            print("Flight recorder doesn't have anything to display right now")
            return

        # now we can print stuff
        rb_data = ''
        if not overlap:
            if append_pos:
                rb_data = self.gen_contents(0, append_pos)
            else:
                rb_data = self.gen_contents(0, size)
        else:
            rb_data = self.gen_contents(append_pos, size)
            rb_data += self.gen_contents(0, append_pos)

        print("{data}".format(data=rb_data))

    # called when invoking 'display_flight_rec'
    def invoke(self, arg, from_tty):
        self.display_flight_rec_contents()

DisplayFlightRecorder()



# encoding=utf-8
# Copyright © 2018 Intel Corporation
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

# Converts a file to a C/C++ #include containing a string

from __future__ import unicode_literals
import argparse
import io
import os


def get_args():
    parser = argparse.ArgumentParser()
    parser.add_argument('input', help="Name of input file")
    parser.add_argument('output', help="Name of output file")
    parser.add_argument("-n", "--name",
                        help="Name of C variable")
    parser.add_argument("-b", "--binary", dest='binary', action='store_const',
                        const=True, default=False)
    args = parser.parse_args()
    return args


def filename_to_C_identifier(n):
    if n[0] != '_' and not n[0].isalpha():
        n = "_" + n[1:]

    return "".join([c if c.isalnum() or c == "_" else "_" for c in n])


def emit_byte(f, b):
    f.write("0x{:02x}, ".format(ord(b)).encode('utf-8'))


def process_file(args):
    with io.open(args.input, "rb") as infile:
        try:
            with io.open(args.output, "wb") as outfile:
                # If a name was not specified on the command line, pick one based on the
                # name of the input file.  If no input filename was specified, use
                # from_stdin.
                if args.name is not None:
                    name = args.name
                else:
                    name = filename_to_C_identifier(args.input)

                outfile.write("static const char {}[] = {{\n".format(name).encode('utf-8'))

                linecount = 0
                while True:
                    byte = infile.read(1)
                    if byte == b"":
                        break

                    if not args.binary:
                        assert(ord(byte) != 0)

                    emit_byte(outfile, byte)
                    linecount = linecount + 1
                    if linecount > 20:
                        outfile.write(b"\n ")
                        linecount = 0
                if not args.binary:
                    outfile.write(b"\n0")
                outfile.write(b"\n};\n\n")
        except Exception:
            # In the event that anything goes wrong, delete the output file,
            # then re-raise the exception. Deleteing the output file should
            # ensure that the build system doesn't try to use the stale,
            # half-generated file.
            os.unlink(args.output)
            raise


def main():
    args = get_args()
    process_file(args)


if __name__ == "__main__":
    main()
